# -*- coding:utf-8 -*-
import boto3
import click
import datetime
import json
import shutil
import subprocess
import os
import platform
import time
import wget


from botocore.exceptions import ClientError
import eulerpublisher.publisher.publisher as pb
from eulerpublisher.publisher import EP_PATH
from eulerpublisher.publisher import OPENEULER_REPO


ROLE_NAME = "vmimport"
SCRIPT_PATH = EP_PATH + "cloudimg/script/"
PACKER_FILE_PATH = EP_PATH + "cloudimg/aws/"
ROLE_POLICY = PACKER_FILE_PATH + "role-policy.json"
TRUST_POLICY = PACKER_FILE_PATH + "trust-policy.json"
DEFAULT_RPMLIST = PACKER_FILE_PATH + "install_packages.txt"
ENA_KO = PACKER_FILE_PATH + "ena.ko"
AWS_DATA_PATH = "/home/tmp/eulerpublisher/cloudimg/data/aws/"


def _check_credentials():
    config = os.path.join(os.environ["HOME"], ".aws", "config")
    credentials = os.path.join(os.environ["HOME"], ".aws", "credentials")
    if not os.path.exists(config) or not os.path.exists(credentials):
        raise TypeError(
            "AWS credentials are not configured,"
            " please run `aws configure` to configure first."
        )


def _check_role(client):
    resp = client.list_roles()
    for role in resp["Roles"]:
        if role["RoleName"] == ROLE_NAME:
            return True
    return False


def _find_object(client, name, bucket):
    resp = client.list_objects(Bucket=bucket)
    for obj in resp["Contents"]:
        if obj["Key"] == name:
            return True
    return False


def _create_policy(bucket=""):
    with open(ROLE_POLICY, "r", encoding="utf-8-sig") as f:
        data = json.load(f)
    data["Statement"][0]["Resource"] = [
        "arn:aws:s3:::" + bucket,
        "arn:aws:s3:::" + bucket + "/*",
    ]
    data["Statement"][1]["Resource"] = [
        "arn:aws:s3:::" + bucket,
        "arn:aws:s3:::" + bucket + "/*",
    ]
    with open(ROLE_POLICY, "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4)


def _resize_cloudimg(arch, version):
    shutil.copy2(ENA_KO, AWS_DATA_PATH)
    script = SCRIPT_PATH + "aws_resize.sh"
    args = [version, arch, AWS_DATA_PATH]
    subprocess.call(["sudo", "sh", script] + args)


def _make_base_image(arch, version):
    if not os.path.exists(AWS_DATA_PATH):
        os.makedirs(AWS_DATA_PATH, exist_ok=True)
    os.chdir(AWS_DATA_PATH)
    raw_file = "openEuler-" + version + "-" + arch + ".raw"
    qcow2_file = "openEuler-" + version + "-" + arch + ".qcow2"
    xz_file = "openEuler-" + version + "-" + arch + ".qcow2.xz"
    if not os.path.exists(raw_file):
        if not os.path.exists(qcow2_file):
            if not os.path.exists(xz_file):
                url = (
                    OPENEULER_REPO
                    + "openEuler-"
                    + version + "/"
                    + "virtual_machine_img"
                    + "/"
                    + arch
                    + "/"
                    + xz_file
                )
                try:
                    click.echo("\nDownloading " + xz_file + "...")
                    wget.download(url)
                except Exception as err:
                    raise click.ClickException("[Prepare] " + str(err))
            subprocess.call(["unxz", "-f", xz_file])
        _resize_cloudimg(arch=arch, version=version)


def _create_packer(arch, region, img_name, source, rpmlist):
    file = PACKER_FILE_PATH + "aws_" + arch + ".json"
    with open(rpmlist, "r") as rpms:
        packages = rpms.read()
    with open(file, "r", encoding="utf-8-sig") as f:
        data = json.load(f)
    data["builders"][0]["region"] = region
    data["builders"][0]["ami_regions"] = [region]
    data["builders"][0]["source_ami"] = source
    data["builders"][0]["ami_name"] = img_name + "-hvm"
    data["provisioners"][0]["environment_vars"] = [
        "INSTALL_PACKAGES=" + packages
    ]
    data["provisioners"][0]["script"] = SCRIPT_PATH + "aws_install.sh"
    with open(file, "w", encoding="utf-8") as f:
        json.dump(data, f, indent=4)


class AwsPublisher(pb.Publisher):
    def __init__(self,
                 version="",
                 arch="",
                 bucket="", 
                 region="",
                 rpmlist=""):
        # 关键参数
        self.img_name = "openEuler-" + version + "-" + arch
        self.version = version
        self.bucket = bucket
        self.region = region
        # 获取支持的架构类型
        if arch != str(platform.machine()):
            raise TypeError(
                "Unsupported architecture " + arch + \
                ", while current host architecture is " + \
                str(platform.machine())
            )
        self.arch = arch
        # 获取要预安装的软件包列表，不显示指定时安装默认包
        if not rpmlist:
            self.rpmlist = DEFAULT_RPMLIST
        else:
            self.rpmlist = os.path.abspath(rpmlist)
        # check aws configure
        _check_credentials()
        # 创建S3客户端
        self.s3_client = boto3.client("s3")
        # 创建IAM客户端
        self.iam_client = boto3.client("iam")
        # 创建EC2客户端
        self.ec2_client = boto3.client("ec2")

    def prepare(self):
        # 获取基础镜像
        _make_base_image(arch=self.arch,
                         version=self.version
        )
        # 上传镜像到S3存储
        key = self.img_name + ".raw"
        if not _find_object(self.s3_client, key, self.bucket):
            click.echo("[Prepare] Uploading raw image to s3 bucket: %s..."
                % self.bucket)
            # 直接使用awscli可以方便实时显示上传进度
            try:
                subprocess.call([
                    "aws",
                    "s3",
                    "cp",
                    AWS_DATA_PATH + key,
                    "s3://" + self.bucket + "/" + key
                ])
            except Exception:
                raise click.ClickException(
                    "[Prepare] Failed to upload raw image to "
                    "s3 bucket: %s" % self.bucket
                )
        else:
            click.echo(
                "[Prepare] %s is already existed on S3://%s, "
                "please delete and re-upload it if you want."
                % (key, self.bucket)
            )
        # 根据实际信息修改role_policy
        _create_policy(self.bucket)
        # 创建vmimport role
        if not _check_role(self.iam_client):
            trust_policy = open(TRUST_POLICY)
            self.iam_client.create_role(
                RoleName=ROLE_NAME,
                AssumeRolePolicyDocument=trust_policy.read()
            )
        # 写入role policy
        role_policy = open(ROLE_POLICY)
        try:
            self.iam_client.put_role_policy(
                RoleName=ROLE_NAME,
                PolicyName=ROLE_NAME,
                PolicyDocument=role_policy.read(),
            )
        except ClientError as e:
            raise click.ClickException(str(e))

        click.echo("[Prepare] finished.\n")

    def build(self):
        # step-1. 导入镜像snapshot
        try:
            resp = self.ec2_client.import_snapshot(
                Description="Import snapshot for " + self.img_name,
                DiskContainer={
                    "Description": self.img_name,
                    "Format": "RAW",
                    "UserBucket": {
                        "S3Bucket": self.bucket,
                        "S3Key": self.img_name + ".raw",
                    },
                },
                RoleName=ROLE_NAME,
            )
        except ClientError as e:
            raise click.ClickException("\n[Build] " + str(e))

        # 循环查询snapshot导入是否完成，完成后获取其ID
        import_taskid = resp["ImportTaskId"]
        status = resp["SnapshotTaskDetail"]["Status"]
        while status != "completed":
            resp = self.ec2_client.describe_import_snapshot_tasks(
                ImportTaskIds=[import_taskid]
            )
            status = \
                resp["ImportSnapshotTasks"][0]["SnapshotTaskDetail"]["Status"]
            time.sleep(2)
        snapshot_id = \
            resp["ImportSnapshotTasks"][0]["SnapshotTaskDetail"]["SnapshotId"]

        # step-2. 根据snapshot_id制作基础AMI镜像
        time_str = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
        if self.arch == "aarch64":
            arch = "arm64"
        else:
            arch = self.arch
        resp = self.ec2_client.register_image(
            Name=self.img_name + "-" + time_str + "-BASE",
            Architecture=arch,
            BlockDeviceMappings=[{
                "DeviceName": "/dev/xvda",
                "Ebs": {"SnapshotId": snapshot_id}
            }],
            Description="This is an middleware used to build final image.",
            EnaSupport=True,
            RootDeviceName="/dev/xvda",
            VirtualizationType="hvm",
        )
        image_id = resp["ImageId"]

        # 3. packer build最终镜像，可通过aws_install.sh定制
        _create_packer(
            arch=self.arch,
            region=self.region,
            img_name=self.img_name + "-" +
                datetime.datetime.now().strftime("%Y%m%d_%H%M%S"),
            source=image_id,
            rpmlist=self.rpmlist
        )
        try:
            subprocess.call([
                "packer",
                "build",
                PACKER_FILE_PATH + "aws_" + self.arch + ".json"
            ])
        except Exception:
            raise click.ClickException("[Build] Failed to build final image.")

        click.echo("[Build] finished\n")



# coding=utf-8
import click
import sys
import os


import eulerpublisher.publisher.publisher as pb
from eulerpublisher.container.container import ContainerPublisher


@click.group(name="container",
             help="Commands for publishing container images")
def group():
    pass


@group.command(name="prepare",
               help="Prepare original materials for building images")
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of container image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-i",
    "--index",
    required=False,
    default="docker_img",
    help="The link index of docker image you want to publish "\
        "from `repo.openeuler.org` to your registry, "\
        "such as `docker_img` or `docker_img/update/current`,"\
        " the default is `docker_img`."
)
def prepare(version, index):
    obj = ContainerPublisher(version=version, index=index)
    ret = obj.prepare()
    if ret != pb.PUBLISH_SUCCESS:
        sys.exit(1)
    else:
        sys.exit(0)


@group.command(name="check",
               help="To test whether the built container image is correct")
@click.option(
    "-p",
    "--repo",
    required=True,
    help="The repository where the tested image is pulled from."
)
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of container image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-g",
    "--registry",
    default="docker.io",
    help="The registry where to push the built container "\
         "image. The default registry is `docker.io`"\
         ", you can choose others such like `quay.io`,"\
         " `hub.oepkgs.net`, etc."
)
def check(repo, version, registry):
    obj = ContainerPublisher(repo=repo, version=version, registry=registry)
    ret = obj.check()
    if ret != pb.PUBLISH_SUCCESS:
        sys.exit(1)
    sys.exit(0)


@group.command(name="push",
               help="To build and push container image to target repo")
@click.option(
    "-p",
    "--repo",
    required=True,
    help="The target repository to push container image to. "
)
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of container image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-g",
    "--registry",
    default="docker.io",
    help="The registry where to push the built container "\
         "image. The default registry is `docker.io`"\
         ", you can choose others such like `quay.io`,"\
         " `hub.oepkgs.net`, etc."
)
@click.option(
    "-f",
    "--dockerfile",
    default="",
    help="The dockerfile to define your image. "\
        "Please enter the path of your dockerfile "\
        "if necessary."
)
def push(repo, version, registry, dockerfile):
    obj = ContainerPublisher(
        repo=repo, 
        version=version,
        registry=registry,
        dockerfile=dockerfile
    )
    ret = obj.build_and_push()
    if ret != pb.PUBLISH_SUCCESS:
        sys.exit(1)
    sys.exit(0)


@group.command(
    name="publish",
    help="To publish container image to target repo"
)
@click.option(
    "-p",
    "--repo",
    default="",
    help="The target repository to push container image to. "\
         "This option is required while option `--multiPublish`"\
         " is not set."
)
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of container image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-g",
    "--registry",
    default="docker.io",
    help="The registry where to push the built container "\
         "image. The default registry is `docker.io`"\
         ", you can choose others such like `quay.io`,"\
         " `hub.oepkgs.net`, etc.. While option `--multiPublish`"\
         " is set, `--registry` is no longer needed."
)
@click.option(
    "-f",
    "--dockerfile",
    default="",
    help="The dockerfile to define your image. "\
        "Please enter the path of your dockerfile "\
        "if necessary."
)
@click.option(
    "-i",
    "--index",
    required=False,
    default="docker_img",
    help="The link index of docker image you want to publish "\
        "from `repo.openeuler.org` to your registry, "\
        "such as `docker_img` or `docker_img/update/current`,"\
        " the default is `docker_img`."
)
@click.option(
    "-m",
    "--mpublish",
    is_flag=True,
    help="This option decides whether to publish the image to "\
        "multiple repositories. "\
        "While this option is set, all target repositories must "\
        "be provided in etc/container/registry.yaml, and the "\
        "options `--repo` and `--registry` are no longer needed."
)
def publish(repo, version, registry, dockerfile, index, mpublish):
    if mpublish:
        multi = True
    else:
        multi = False
    obj = ContainerPublisher(
        repo=repo,
        version=version,
        registry=registry,
        dockerfile=dockerfile,
        index=index,
        multi=multi
    )
    ret = obj.publish()
    if ret != pb.PUBLISH_SUCCESS:
        sys.exit(1)
    sys.exit(0)



# -*- coding:utf-8 -*-
import click
import datetime
import subprocess
import os
import platform
import wget


import eulerpublisher.publisher.publisher as pb
from eulerpublisher.publisher import EP_PATH
from eulerpublisher.publisher import OPENEULER_REPO


GEN_DATA_PATH = "/home/tmp/eulerpublisher/cloudimg/data/gen/"
DEFAULT_RPMLIST = EP_PATH + "cloudimg/gen/install_packages.txt"
SCRIPT_PATH = EP_PATH + "cloudimg/script/"


class GenPublisher(pb.Publisher):
    def __init__(self,
                 version="",
                 arch="",
                 rpmlist="",
                 output=""):
        # 关键参数
        self.version = version
        # 获取支持的架构类型
        if arch != str(platform.machine()):
            raise TypeError(
                "Unsupported architecture " + arch + \
                "while current host architecture is " + \
                str(platform.machine())
            )
        self.arch = arch
        # 获取要预安装的软件包列表，不显示指定时安装默认包
        if not rpmlist:
            self.rpmlist = DEFAULT_RPMLIST
        else:
            self.rpmlist = os.path.abspath(rpmlist)
        # 构建生成的镜像名
        self.output = output
    
    def prepare(self):
        if not os.path.exists(GEN_DATA_PATH):
            os.makedirs(GEN_DATA_PATH, exist_ok=True)
        os.chdir(GEN_DATA_PATH)
        qcow2_file = "openEuler-" + self.version + "-" + self.arch + ".qcow2"
        xz_file = "openEuler-" + self.version + "-" + self.arch + ".qcow2.xz"
        if not os.path.exists(qcow2_file):
            if not os.path.exists(xz_file):
                url = (
                    OPENEULER_REPO
                    + "openEuler-"
                    + self.version + "/"
                    + "virtual_machine_img"
                    + "/"
                    + self.arch
                    + "/"
                    + xz_file
                )
                try:
                    click.echo("\nDownloading " + xz_file + "...")
                    wget.download(url)
                except Exception as err:
                    raise click.ClickException("[Prepare] " + str(err))
            subprocess.call(["unxz", "-f", xz_file])
        click.echo("\n[Prepare] finsihed")
    
    def build(self):
        input = "openEuler-" + self.version + "-" + self.arch + ".qcow2"
        time_str = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
        # 默认输出保存为带有时间信息的qcow2格式
        if not self.output:
            self.output = "openEuler-" + self.version + "-" + self.arch \
                     + "-" + time_str + ".qcow2"
        script = SCRIPT_PATH + "gen_install.sh"
        args = [input, self.output, GEN_DATA_PATH, self.rpmlist]
        subprocess.call(["sudo", "sh", script] + args)
        click.echo("\n[Build] finsihed")
        return pb.PUBLISH_SUCCESS



# coding=utf-8
import click

from eulerpublisher.cloudimg.gen.gen import GenPublisher


@click.group(
    name="gen",
    help="Commands for building generic cloud images"
)
def group():
    pass


@group.command(
    name="prepare",
    help="Prepare original materials for building generic cloud image"
)
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of cloud image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-a",
    "--arch",
    required=True,
    help="The architecture of cloud image, "\
        "limited to `x86_64` or `aarch64`."
)
def prepare(version, arch):
    obj = GenPublisher(version=version, arch=arch)
    obj.prepare()


@group.command(name="build", help="Build generic cloud image.")
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of cloud image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-a",
    "--arch",
    required=True,
    help="The architecture of cloud image, "\
        "limited to `x86_64` or `aarch64`."
)
@click.option(
    "-p",
    "--rpmlist",
    help="The packages you want to install."
)
@click.option(
    "-o",
    "--output",
    help="The name of built cloud image."
)
def build(version, arch, rpmlist, output):
    obj = GenPublisher(version=version,
                       arch=arch,
                       rpmlist=rpmlist,
                       output=output)
    obj.build()


# coding=utf-8
import click

from eulerpublisher.cloudimg.aws.aws import AwsPublisher


@click.group(name="aws", help="Commands for publishing AMI")
def group():
    pass


@group.command(name="prepare",
               help="Prepare original materials for building AMI")
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of cloud image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-a",
    "--arch",
    required=True,
    help="The architecture of cloud image, "\
        "limited to `x86_64` or `aarch64`."
)
@click.option(
    "-b",
    "--bucket",
    required=True,
    help="The bucket_name identifier."
)
def prepare(version, arch, bucket):
    obj = AwsPublisher(version=version,
                       arch=arch,
                       bucket=bucket)
    obj.prepare()


@group.command(name="build", help="Build AMI for AWS.")
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of cloud image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-a",
    "--arch",
    required=True,
    help="The architecture of cloud image, "\
        "limited to `x86_64` or `aarch64`."
)
@click.option(
    "-p",
    "--rpmlist",
    help="The packages you want to install."
)
@click.option(
    "-b",
    "--bucket",
    required=True,
    help="The bucket_name identifier."
)
@click.option(
    "-r",
    "--region",
    required=True,
    help="The AWS region, such as `ap-northeast-2`."
)
def build(version, arch, rpmlist, bucket, region):
    obj = AwsPublisher(version=version,
                       arch=arch,
                       rpmlist=rpmlist,
                       bucket=bucket,
                       region=region)
    obj.build()


@group.command(name="publish", help="Make AMI for AWS.")
@click.option(
    "-v",
    "--version",
    required=True,
    help="The version of cloud image, "\
        "such as 22.03-LTS, 22.03-LTS-SP1, etc."
)
@click.option(
    "-a",
    "--arch",
    required=True,
    help="The architecture of cloud image, "\
        "limited to `x86_64` or `aarch64`."
)
@click.option(
    "-p",
    "--rpmlist",
    help="The packages you want to install."
)
@click.option(
    "-b",
    "--bucket",
    required=True,
    help="The bucket_name identifier."
)
@click.option(
    "-r",
    "--region",
    required=True,
    help="The AWS region, such as `ap-northeast-2`."
)
def publish(version, arch, rpmlist, bucket, region):
    obj = AwsPublisher(version=version,
                       arch=arch,
                       rpmlist=rpmlist,
                       bucket=bucket,
                       region=region)
    obj.prepare()
    obj.build()


# coding=utf-8
import os


EP_PATH = None
OPENEULER_REPO = "http://repo.openeuler.org/"

paths = [
    os.path.dirname(os.__file__) + '/' + '../../etc/eulerpublisher/',
    '/usr/local/etc/eulerpublisher/',
    '/usr/etc/eulerpublisher',
    '/etc/eulerpublisher/'
]

for path in paths:
    if os.path.exists(path):
        EP_PATH = path
        break


# coding=utf-8

# global varibles
PUBLISH_FAILED = 1
PUBLISH_SUCCESS = 0
ARCHS = ["x86_64", "aarch64"]


# parent `publisher`
class Publisher:

    def __init__():
        pass

    # prepare stage
    def prepare():
        pass

    # `build` method that can be executed separately
    def build():
        pass

    # `push` method that can be executed separately
    def push():
        pass

    # The situation where `build` and `push` must be
    # executed at the same time
    def build_and_push():
        pass

    # One-click publishing
    def publish():
        pass



# coding=utf-8
import click
import datetime
import docker
import os
import platform
import re
import shutil
import subprocess
import wget
import yaml
import time


import eulerpublisher.publisher.publisher as pb
from eulerpublisher.publisher import EP_PATH
from eulerpublisher.publisher import OPENEULER_REPO

CACHE_DATA_PATH = "/tmp/eulerpublisher/container/"
DOCKERFILE_PATH = EP_PATH + "container/Dockerfile"
TAGS = EP_PATH + "container/tags.yaml"
REGISTRY = EP_PATH + "container/registry.yaml"

# max time(sencond) to start docker
START_TIMEOUT = 300
# times to retry download
RETRY_DOWNLOAD = 10


# tag image
def _get_tags(repository, version, multi=False):
    repo_list = []
    if not multi:
        repo_list.append(repository)
    else:
        with open(REGISTRY, 'r') as f:
            env = yaml.safe_load(f)
        for key in env:
            repo_list.append(str(env[key][2]))
    # tag image for all registries
    tags = ""
    for repo in repo_list:
        tags += "-t " + repo + ":" + version.lower()
        with open(TAGS, 'r') as f:
            data = yaml.safe_load(f)
        version = version.upper()
        if version in data:
            for tag in data[version]:
                tags += " -t " + repo + ":" + str(tag)
        tags += " "
    return tags


# login registry
def _login_registry(registry, multi=False):
    # login single registry
    if not multi:
        if subprocess.call("echo %s | docker login --username %s "
            "--password-stdin %s" % (\
            os.environ["LOGIN_PASSWORD"], \
            os.environ["LOGIN_USERNAME"], \
            registry), \
            shell=True) != 0:
            return pb.PUBLISH_FAILED
        return pb.PUBLISH_SUCCESS
    # login multiple registries in registry.yaml
    with open(REGISTRY, 'r') as f:
        env = yaml.safe_load(f)
 
    '''
    1. Login all taget registries.
    2. Before logging in, all users and their passwords must
       be set into the environment
    '''
    for key in env: 
        username = os.environ[str(env[key][0])]
        password = os.environ[str(env[key][1])]
        if subprocess.call("echo %s | docker login --username %s "
            "--password-stdin %s" % (password, username, str(key)), \
            shell=True) != 0:
            return pb.PUBLISH_FAILED
    return pb.PUBLISH_SUCCESS


# Run container and compare the result with 'param'
def _run(tag="", cmd="", param=""):
    ret = pb.PUBLISH_FAILED
    try:
        client = docker.from_env()
        container = client.create_container(image=tag, command=cmd, detach=True)
        client.start(container)
        subprocess.call("docker logs " + container["Id"] + " >>logs.txt",
                        shell=True)
        logs = open("logs.txt")
        for line in logs:
            if param in line:
                ret = pb.PUBLISH_SUCCESS
        logs.close()
        subprocess.call(["rm", "-rf", "logs.txt"])
        client.stop(container)
    except docker.errors.DockerException as err:
        click.echo(click.style(f"[run_container] {err}", fg="red"))
    return ret


# Start docker
def _start_docker():
    cnt = 0
    if subprocess.call(['docker', 'info'], stdout = subprocess.PIPE, \
            stderr = subprocess.STDOUT):
        syst = platform.system()
        if syst == "Darwin":
            subprocess.call(['open', '-g', '-a', 'Docker'], \
                stdout = subprocess.PIPE, stderr = subprocess.STDOUT)
        elif syst == "Linux":
            subprocess.call(['systemctl', 'start', 'docker'], \
                stdout = subprocess.PIPE, stderr = subprocess.STDOUT)
        else:
            click.echo(click.style('\nUnsupported system %s.' \
                % syst, fg="red"))
        click.echo('Starting docker ...')
    while subprocess.call(['docker', 'info'], stdout = subprocess.PIPE, \
            stderr = subprocess.STDOUT):
        if cnt >= START_TIMEOUT:
            click.echo(click.style('\nFailed to start docker, ' \
                'please ensure the docker is installed.', fg="red"))
            return pb.PUBLISH_FAILED
        time.sleep(5)
        cnt += 5
    return pb.PUBLISH_SUCCESS


def _download(url):
    retry_cnt = 0
    while retry_cnt < RETRY_DOWNLOAD:
        try:
            wget.download(url)
            return pb.PUBLISH_SUCCESS
        except Exception as err:
            click.echo(click.style(f"\nTry {retry_cnt+1}/{RETRY_DOWNLOAD}"\
                " failure: " + str(err), fg="red"))
            retry_cnt += 1
    return pb.PUBLISH_FAILED


# Class for publishing container images
class ContainerPublisher(pb.Publisher):
    def __init__(self,
                 repo="",
                 version="",
                 registry="",
                 index="",
                 dockerfile="",
                 multi=False):
        self.repo = repo
        self.version = version
        self.registry = registry
        self.index = index
        self.multi = multi
        if dockerfile:
            self.dockerfile = os.path.abspath(dockerfile)
        else:
            self.dockerfile = ""
        

    def prepare(self):
        os.makedirs(CACHE_DATA_PATH, exist_ok=True)
        os.chdir(CACHE_DATA_PATH)
        os.makedirs(self.version, exist_ok=True)
        shutil.copy2(DOCKERFILE_PATH, self.version + "/Dockerfile")
        os.chdir(self.version)

        for arch in pb.ARCHS:
            if arch == "x86_64":
                docker_arch = "amd64"
            elif arch == "aarch64":
                docker_arch = "arm64"
            # download base images
            index = "openEuler-" + self.version.upper() + "/" + \
                self.index + "/" + arch + "/"
            file_name = "openEuler-docker." + arch + ".tar.xz"
            if not os.path.exists(file_name): 
                download_url = OPENEULER_REPO + index + file_name
                if _download(download_url) != pb.PUBLISH_SUCCESS:
                    return pb.PUBLISH_FAILED
                click.echo(
                    "\n[Prepare] Download %s successfully." % file_name
                )
            # check
            sha256sum = file_name + ".sha256sum"
            subprocess.call(["rm", "-rf", sha256sum])
            sha256sum_url = OPENEULER_REPO + index + sha256sum
            if _download(sha256sum_url) != pb.PUBLISH_SUCCESS:
                return pb.PUBLISH_FAILED
            click.echo("\n[Prepare] Download %s successfully." % sha256sum)
            subprocess.call(["shasum", "-c", sha256sum])
            # get rootfs
            rootfs = "openEuler-docker-rootfs." + docker_arch + ".tar.xz"
            if os.path.exists(rootfs):
                continue
            tar_cmd = [
                "tar",
                "-xf",
                file_name,
                "--exclude",
                "layer.tar",
            ]
            subprocess.call(tar_cmd)
            for file in os.listdir("."):
                if file.endswith(".tar") and not re.search("openEuler", file):
                    os.rename(file, "openEuler-docker-rootfs."
                              + docker_arch + ".tar")
                    subprocess.call([
                        "xz", "-z",
                        "openEuler-docker-rootfs." + docker_arch + ".tar"
                    ])
        click.echo("[Prepare] finished")
        return pb.PUBLISH_SUCCESS

    def build_and_push(self):
        try:
            os.chdir(CACHE_DATA_PATH)
            # to build with Dockerfile
            if os.path.exists(self.dockerfile):
                shutil.copy2(self.dockerfile, self.version + "/Dockerfile")
            # ensure qemu is installed so that it can build multi-platform images
            if subprocess.call(["qemu-img", "--version"], \
                    stdout = subprocess.PIPE,stderr = subprocess.STDOUT) != 0:
                click.echo(click.style(
                        "\n[Prepare] please install qemu first, \
                        you can use command <yum install qemu-img>.",
                        fg="red"
                ))
                return pb.PUBLISH_FAILED
            # ensure the docker is starting
            if _start_docker() != pb.PUBLISH_SUCCESS:
                return pb.PUBLISH_FAILED
            # login registry
            if _login_registry(self.registry, self.multi) != pb.PUBLISH_SUCCESS:
                return pb.PUBLISH_FAILED
            # build mutil-platform images with 'buildx'
            builder_name = "euler_builder_" + datetime.datetime.now().strftime(
                "%Y%m%d_%H%M%S"
            )
            if subprocess.call([
                "docker", "buildx", "create", "--use", "--name", builder_name
            ]) != 0:
                return pb.PUBLISH_FAILED
            # build and push docker image
            os.chdir(self.version)
            tags = _get_tags(repository=self.registry + "/" + self.repo,
                             version=self.version,
                             multi=self.multi)
            if subprocess.call(
                "docker buildx build " + \
                "--platform linux/arm64,linux/amd64 " + \
                tags + \
                " --push " + \
                ".",
                shell=True
            )!= 0:
                return pb.PUBLISH_FAILED
            subprocess.call(["docker", "buildx", "stop", builder_name])
            subprocess.call(["docker", "buildx", "rm", builder_name])
        except (OSError, subprocess.CalledProcessError) as err:
            click.echo(click.style(f"[Build and Push] {err}", fg="red"))
        click.echo("[Build and Push] finished")
        return pb.PUBLISH_SUCCESS

    # To test the new image，you should first pull it
    def check(self):
        result = pb.PUBLISH_SUCCESS
        try:
            client = docker.from_env()
            if self.registry == "docker.io":
                image_tag = self.repo + ":" + self.version
            else:
                image_tag = self.registry + "/" + self.repo + ":" + self.version
            image = client.images(name=image_tag)
            if image == []:
                click.echo("Pulling " + image_tag + "...")
                client.pull(image_tag)
            # check basic information of new image
            image = client.images(name=image_tag)
            image_info = client.inspect_image(image[0]["Id"])
            for tag in image[0]["RepoTags"]:
                if tag == image_tag:
                    # check OS type
                    if image_info["Os"] != "linux":
                        click.echo(click.style(
                            "[Check] OS type <%s> is unknown." % image_info["Os"],
                            fg="red"
                        ))
                        result = pb.PUBLISH_FAILED
                    # check platform type
                    if image_info["Architecture"] != "amd64" and \
                    image_info["Architecture"] != "arm64":
                        click.echo(click.style(
                            "[Check] Architecture <%s> is not expected." %
                            image_info["Architecture"],
                            fg="red"
                        ))
                        result = pb.PUBLISH_FAILED

            # test time zone settings
            if _run(tag=image_tag, cmd="date", param="UTC") != pb.PUBLISH_SUCCESS:
                click.echo(click.style(
                    "[Check] time zone setting is not UTC", fg="red"
                ))
                result = pb.PUBLISH_FAILED
        except docker.errors.DockerException as err:
            click.echo(click.style(f"[Check] {err}", fg="red"))
        click.echo("[Check] finished")
        return result

    # Publish with one click
    def publish(self):
        click.echo("\n[Publish] start to publish openEuler-" + self.version + '...')
        if self.prepare() != pb.PUBLISH_SUCCESS:
            click.echo(click.style(
                "[Publish] Download failed.", fg="red"
            ))
            return pb.PUBLISH_FAILED
        if self.build_and_push() != pb.PUBLISH_SUCCESS:
            click.echo(click.style(
                "[Publish] Build and push failed.", fg="red"
            ))
            return pb.PUBLISH_FAILED
        click.echo("[Publish] finished")
        return pb.PUBLISH_SUCCESS



import click

from eulerpublisher.cloudimg.gen import cli as gen_cli
from eulerpublisher.cloudimg.aws import cli as aws_cli


@click.group(name="cloudimg", help="Commands for publishing cloud images")
def group():
    pass


# Unified interface for extension.
group.add_command(aws_cli.group)
group.add_command(gen_cli.group)



# coding=utf-8
import click

from eulerpublisher.container import cli as container_cli
from eulerpublisher.cloudimg import cli as cloudimg_cli


@click.group()
def entrance():
    pass


def _add_commands():
    # Unified interface for extension.
    entrance.add_command(container_cli.group)
    entrance.add_command(cloudimg_cli.group)


def main():
    _add_commands()
    entrance()


if __name__ == "__main__":
    main()
